@@ -17,6 +17,7 @@ import functools
 from multiprocessing.dummy import Array
 import re
 from types import FunctionType
+from rest_framework.request import Request
 from rest_framework.response import Response
 from TDhelper.generic.classDocCfg import doc
 from TDhelper.generic.dictHelper import createDictbyStr, findInDict
@@ -42,8 +43,6 @@ class Event:
         self._httpHeaders["api-token"] = secret if secret else ""
 
     def Register(self, serviceClass=None):
-        if not serviceClass:
-            serviceClass = type(self)
         for k, v in serviceClass.__dict__.items():
             if isinstance(v, FunctionType):
                 if not k.startswith("__"):
@@ -84,7 +83,10 @@ class Event:
                                     params_str[var_name] = {"type": "<class 'dict'>"}
                                 else:
                                     params_str[var_name] = {"type": "<class 'any'>"}
-                            for var_name, var_value in v.__wrapped__.__annotations__.items():
+                            for (
+                                var_name,
+                                var_value,
+                            ) in v.__wrapped__.__annotations__.items():
                                 params_str[var_name]["type"] = str(var_value)
                             if v.__wrapped__.__defaults__:
                                 for i in range(len(parmas_tmp), 0, -1):
@@ -104,7 +106,7 @@ class Event:
                             post_data["params"] = params_str
                         else:
                             post_data["params"] = config["handle"]["params"]
-                        #post_data = serializePostData(post_data)
+                        # post_data = serializePostData(post_data)
                         state, body = POST(
                             self._event_uri + "handle/",
                             post_data=post_data,
@@ -119,7 +121,9 @@ class Event:
                                 logging.info("handle(%s) register SUCCESS" % k)
                                 handleId = ret["msg"]["id"]
                                 if "trigger" in config:
-                                    self._triggerRegister(k, handleId, config["trigger"])
+                                    self._triggerRegister(
+                                        k, handleId, config["trigger"]
+                                    )
                             else:
                                 logging.error(
                                     "register handle %s is error. %s" % (k, ret["msg"])
@@ -421,20 +425,50 @@ class Manager:
                 value = relation["default"]
         else:
             if not sources:
-                if len(m_key) == 2:
+                if len(m_key) >= 2:
                     if m_key[0].lower() == "args":
-                        if int(m_key[1]) < len(args):
-                            value = args[int(m_key[1])]
-                        else:
+                        if len(m_key) != 2:
                             if "default" not in relation:
                                 value = None
                             else:
                                 value = relation["default"]
+                        else:
+                            if int(m_key[1]) < len(args):
+                                value = args[int(m_key[1])]
+                            else:
+                                if "default" not in relation:
+                                    value = None
+                                else:
+                                    value = relation["default"]
                     elif m_key[0].lower() == "kwargs":
-                        if m_key[1] in kwargs:
-                            value = kwargs[m_key[1]]
+                        try:
+                            find_key = relation["key"].lstrip(r"kwargs").lstrip(".")
+                            value = findInDict(find_key, kwargs)
+                        except Exception as e:
+                            if "default" not in relation:
+                                logging.error(e.args)
+                                value = None
+                            else:
+                                value = relation["default"]
+                    elif m_key[0].lower() == "request":
+                        m_context = None
+                        for o in args:
+                            if isinstance(o, Request):
+                                m_context = o
+                                break
+                        if m_context:
+                            try:
+                                find_key = relation["key"].lstrip("request").lstrip(".")
+                                value = findInDict(find_key, m_context)
+                            except Exception as e:
+                                if "default" not in relation:
+                                    logging.error(e.args)
+                                    value = None
+                                else:
+                                    value = relation["default"]
                         else:
                             if "default" not in relation:
+                                logging.error("not found context 'request'.")
                                 value = None
                             else:
                                 value = relation["default"]
@@ -449,20 +483,28 @@ class Manager:
                     else:
                         value = relation["default"]
                 if isLocation:
-                    if m_key:
-                        if m_key[0] == "args":
-                            back_ret["args"].append(value)
-                        elif m_key[0] == "kwargs":
-                            back_ret["kwargs"][key] = value
+                    mapping_key = key.split(".")
+                    if len(mapping_key) == 2:
+                        if mapping_key[0].lower() == "args":
+                            if int((mapping_key[1])) < len(back_ret["args"]):
+                                back_ret["args"].insert(int(mapping_key[1]), value)
+                            else:
+                                back_ret["args"].append(value)
+                        elif mapping_key[0].lower() == "kwargs":
+                            back_ret["kwargs"][mapping_key[1]] = value
                         else:
-                            back_ret["kwargs"][key] = value
+                            if int((mapping_key[1])) < len(back_ret["args"]):
+                                back_ret["args"].insert(int(mapping_key[1]), value)
+                            else:
+                                back_ret["args"].append(value)
                     else:
-                        back_ret["kwargs"][key] = value
+                        back_ret["args"].append(value)
                 else:
                     back_ret[key] = value
             else:
                 if "master_func_results" in kwargs:
                     m_func_result = kwargs["master_func_results"]
+                    relation_key = key.lstrip("self").lstrip(".")
                     if isinstance(m_func_result, Response):
                         m_func_result = m_func_result.data
                     if isinstance(m_func_result, (tuple, list)):
@@ -482,37 +524,41 @@ class Manager:
                         if key == "self":
                             value = m_func_result
                         else:
-                            result_key = key.split(".")
-                            if len(result_key) == 2:
-                                if result_key[1] in m_func_result:
-                                    value = m_func_result[result_key[1]]
-                                else:
+                            try:
+                                value = findInDict(relation_key, m_func_result)
+                            except Exception as e:
+                                if "default" not in relation:
+                                    logging.error(e.args)
                                     value = None
-                            else:
-                                value = None
+                                else:
+                                    value = relation["default"]
                     else:
                         # master func results is object
                         if key == "self":
                             value = m_func_result
                         else:
-                            result_key = key.split(".")
-                            for i in range(1, len(result_key)):
-                                m_func_result = getattr(m_func_result, result_key[i])
-                            value = m_func_result
+                            try:
+                                value = findInDict(relation_key, m_func_result)
+                            except Exception as e:
+                                if "default" not in relation:
+                                    logging.error(e.args)
+                                    value = None
+                                else:
+                                    value = relation["default"]
                     # mapping params
                     if isLocation:
                         if len(m_key) == 2:
                             if m_key[0].lower() == "args":
                                 if int(m_key[1]) < len(back_ret["args"]):
-                                    back_ret["args"][int(m_key[1])] = value
+                                    back_ret["args"].insert(int(m_key[1]), value)
                                 else:
-                                    back_ret["kwargs"][m_key[0]] = value
+                                    back_ret["args"].append(value)
                             else:
-                                back_ret["kwargs"][m_key[0]] = value
+                                back_ret["kwargs"][m_key[1]] = value
                         else:
-                            back_ret["kwargs"][m_key[0]] = value
+                            back_ret["kwargs"][m_key] = value
                     else:
-                        back_ret[m_key[0]] = value
+                        back_ret[m_key[1]] = value
         return back_ret
 
     def _generic_params(self, mapping: dict, isLocation=False, *args, **kwargs):
@@ -527,13 +573,16 @@ class Manager:
         ret = {}
         if isLocation:
             ret = {"args": [], "kwargs": {}}
-        try:
-            mapping = json.loads(mapping, encoding="utf-8")
-        except Exception as e:
-            logging.error(e)
-            mapping = {}
+        if isinstance(mapping, str):
+            try:
+                mapping = json.loads(mapping, encoding="utf-8")
+            except Exception as e:
+                logging.error(e)
+                mapping = {}
         for k, v in mapping.items():
             self._set_params_mapping(ret, isLocation, k, v, *args, **kwargs)
+        if isLocation:
+            ret["args"] = tuple(ret["args"])
         return ret
 
     def _location_call(self, item, *args, **kwargs):
