@@ -21,14 +21,12 @@ class register:
         self._httpHeaders = httpHeaders
         self._httpHeaders["api-token"] = self._secret
 
-    def Register(self, serviceClass=None):
+    def Register(self, serviceClass:type=None):
         """
         使用方法描述进行注册
         - params:
         -   serviceClass: <class>, 类
         """
-        if not serviceClass:
-            serviceClass = type(self)
         for k, v in serviceClass.__dict__.items():
             if isinstance(v, FunctionType):
                 if not k.startswith("__"):
@@ -116,6 +114,13 @@ class register:
 
 
 class perACL:
+    '''权限检查
+    args:
+        rpc_key: permission key.
+        params_container_class: context class.
+        platformKey: platform key.
+        tokenKey: token key filed.
+    '''
     def __init__(
         self, rpc_key, params_container_class, platformKey=None, tokenKey="usr-token"
     ):
